@@ -1,16 +1,15 @@
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-import pkg_resources
+from __future__ import absolute_import, division, print_function
+
 import warnings
 
 import numpy as np
 import pandas as pd
+import pkg_resources
+import textwrap
 
 from ..core.pycompat import basestring
 from ..core.utils import is_scalar
 
-
 ROBUST_PERCENTILE = 2.0
 
 
@@ -23,6 +22,7 @@ def _load_default_cmap(fname='default_colormap.csv'):
     # Not sure what the first arg here should be
     f = pkg_resources.resource_stream(__name__, fname)
     cm_data = pd.read_csv(f, header=None).values
+    f.close()
 
     return LinearSegmentedColormap.from_list('viridis', cm_data)
 
@@ -161,7 +161,7 @@ def _determine_cmap_params(plot_data, vmin=None, vmax=None, cmap=None,
     """
     import matplotlib as mpl
 
-    calc_data = np.ravel(plot_data[~pd.isnull(plot_data)])
+    calc_data = np.ravel(plot_data[np.isfinite(plot_data)])
 
     # Handle all-NaN input data gracefully
     if calc_data.size == 0:
@@ -298,7 +298,7 @@ def _infer_xy_labels_3d(darray, x, y, rgb):
     assert rgb is not None
 
     # Finally, we pick out the red slice and delegate to the 2D version:
-    return _infer_xy_labels(darray.isel(**{rgb: 0}).squeeze(), x, y)
+    return _infer_xy_labels(darray.isel(**{rgb: 0}), x, y)
 
 
 def _infer_xy_labels(darray, x, y, imshow=False, rgb=None):
@@ -355,3 +355,24 @@ def get_axis(figsize, size, aspect, ax):
         ax = plt.gca()
 
     return ax
+
+
+def label_from_attrs(da):
+    ''' Makes informative labels if variable metadata (attrs) follows
+        CF conventions. '''
+
+    if da.attrs.get('long_name'):
+        name = da.attrs['long_name']
+    elif da.attrs.get('standard_name'):
+        name = da.attrs['standard_name']
+    elif da.name is not None:
+        name = da.name
+    else:
+        name = ''
+
+    if da.attrs.get('units'):
+        units = ' [{}]'.format(da.attrs['units'])
+    else:
+        units = ''
+
+    return '\n'.join(textwrap.wrap(name + units, 30))
